@@ -3,14 +3,14 @@ BEGIN {
   $POE::Component::IRC::Plugin::NickServID::AUTHORITY = 'cpan:HINRIK';
 }
 BEGIN {
-  $POE::Component::IRC::Plugin::NickServID::VERSION = '6.39';
+  $POE::Component::IRC::Plugin::NickServID::VERSION = '6.40';
 }
 
 use strict;
 use warnings FATAL => 'all';
 use Carp;
 use POE::Component::IRC::Plugin qw( :ALL );
-use POE::Component::IRC::Common qw( u_irc );
+use POE::Component::IRC::Common qw( u_irc parse_user );
 
 sub new {
     my ($package) = shift;
@@ -25,7 +25,7 @@ sub PCI_register {
     my ($self, $irc) = @_;
     $self->{nick} = $irc->{nick};
     $self->{irc} = $irc;
-    $irc->plugin_register($self, 'SERVER', qw(004 nick));
+    $irc->plugin_register($self, 'SERVER', qw(isupport nick));
     return 1;
 }
 
@@ -33,7 +33,10 @@ sub PCI_unregister {
     return 1;
 }
 
-sub S_004 {
+# we identify after S_isupport so that pocoirc has a chance to turn on
+# CAPAB IDENTIFY-MSG (if the server supports it) before the AutoJoin
+# plugin joins channels
+sub S_isupport {
     my ($self, $irc) = splice @_, 0, 2;
     $irc->yield(nickserv => "IDENTIFY $self->{Password}");
     return PCI_EAT_NONE;
@@ -56,8 +59,7 @@ sub S_nick {
 
 =head1 NAME
 
-POE::Component::IRC::Plugin::NickServID - A PoCo-IRC plugin
-which identifies with FreeNode's NickServ when needed
+POE::Component::IRC::Plugin::NickServID - A PoCo-IRC plugin which identifies with NickServ when needed
 
 =head1 SYNOPSIS
 
